#ifdef __cplusplus
extern "C" {
#endif

#include "stdlib.h"
#include "keyboard.hpp"
#include "keyboard_syscalls.h"
#include "disp_tools.hpp"
#include "display_tools.h"
#include "display_syscalls.h"
#include "app_syscalls.h"
#include "convert_syscalls.h"
#include "system_syscalls.h"
#include "heap_syscalls.h"
#include "RTC_syscalls.h"
#include "STD_syscalls.h"
#include "convert_tools.h"
#include "mcs_syscalls.h"
#include "memview2.hpp"
#include "fxCG_display.h"
#include "misc_tools.h"


// new syscalls
const int SysCallWrapper[] = {0xD201422B,0x60F20000,0x80020070};
int (*iSysCallFuncPtr)( int R4, int R5, int R6, int R7, int FNo ) = (  int (*)(int,int,int,int,int))&SysCallWrapper;
#define PutKey(key) (void)(*iSysCallFuncPtr)( key,key<256, 0, 0, 0x0EA9)
#define Alpha_GetData(VarName,Dest) (void)(*iSysCallFuncPtr)( VarName,(int)Dest, 0, 0, 0x0034)
#define Alpha_ClearAll() (void)(*iSysCallFuncPtr)( 0,0, 0, 0, 0x0036)
#define App_PRGM() (void)(*iSysCallFuncPtr)( 0,0, 0, 0, 0x1945)
#define Timer_Start(ID) (int)(*iSysCallFuncPtr)( ID,0, 0, 0, 0x08DB)
#define Timer_Stop(ID) (int)(*iSysCallFuncPtr)( ID,0, 0, 0, 0x08DC)
#define Timer_Install(ID, handler, elapse) (int)(*iSysCallFuncPtr)( ID, (int)handler, elapse, 0, 0x08D9)
#define Timer_Deinstall(ID) (int)(*iSysCallFuncPtr)( ID,0, 0, 0, 0x08DA)

//250 ms timer
#define TimerInterval 250 
//iDelay is actually a wait-state counter
int iDelay = 1000/TimerInterval;
//index to the text for replaying
int iIndex = 0;


/*text to be replayed, contains control codes + following script
"Hello World!"[EXE]
"This is basic program"[EXE]
"written & run by TSR"[EXE]
"Press [EXE] to quit"[EXE]
"Rotating"_(disp)
1->A
*/
const short Text[] = {KEY_CTRL_F3,'T','S','R','D','E','M','O',KEY_CTRL_EXE,KEY_CTRL_F2,
'"','H','e','l','l','o',' ','W','o','r','l','d','!','"',KEY_CTRL_EXE,
'"','T','h','i','s',' ','i','s',' ','b','a','s','i','c',' ','p','r','o','g','r','a','m','"',KEY_CTRL_EXE,
'"','w','r','i','t','t','e','n',' ','&',' ','r','u','n',' ','b','y',' ','T','S','R','"',KEY_CTRL_EXE,
'"','P','r','e','s','s',' ','[','E','X','E',']',' ','t','o',' ','q','u','i','t','"',KEY_CTRL_EXE,
'"','R','o','t','a','t','e','i','n','g','"',12,'1',KEY_CHAR_STORE,'A',KEY_CTRL_EXIT,KEY_CTRL_EXE};
//buffer for A variable
char variable[24]={0};
//buffer for scrolling
//unsigned char Screen[1024+16];
int lastchar  = 0;

int SetTimer( int ID, int elapse, void *handler ){  // Simon's HTML help
  int result;
   result = Timer_Install( ID+5, handler, elapse );
   if (result==ID+5){ result = Timer_Start( ID+5 );}
 return result;
}

int KillTimer( int ID ){                           // Simon's HTML help
  int result;
   result = Timer_Stop( ID+5 );
   if (result==ID+5){ result = Timer_Deinstall( ID+5 );}
 return result;
}

void SetCGQuitHandler(void *handler){
 *(int *) 0x810043C = (int)handler;  // 0x8100298??,  seems same as fx9860 ?
}

//timer handler worth simple state machine
void TimerHandler(void)
{
	short c = Text[iIndex];
	if(iDelay)
	{
		//wait this round
		iDelay--;
		return;
	}
	if(c)
	{
		//inject key
		PutKey(c);iIndex++;
		if(!Text[iIndex])
		{
			iDelay = 1000/TimerInterval;
		}
		return;
	}
	//keys are gone, check A
	Alpha_GetData('A',variable);
	if(variable[0])
	{
		KillTimer(1);
	}
	else
	{
		//scroll screen 1 pixel row up
		//Bdisp_GetDisp_DD(Screen);
		//memcpy(Screen+1024-16,Screen,16);
		//DrawAll(Screen+16);
                locate_OS( 7, 23 );
                if (lastchar == 0) Print("/");
                if (lastchar == 1) Print("-");
                if (lastchar == 2) Print("\\");
                ++lastchar;
                if (lastchar >= 3){
                 lastchar  = 0;
                }
	}
}

void QuitHandler(void)
{
	KillTimer(1);
}

void G3A_main(void)
{
	SetCGQuitHandler(&QuitHandler);
	Alpha_ClearAll();
	SetTimer(1,TimerInterval, &TimerHandler);

	App_PRGM();// terminate and stay resident :-)
	return ;
}



#pragma section _BR_Size
unsigned long BR_Size;
#pragma section

#ifdef __cplusplus
}
#endif

